"""
idem-azure can autogenerate states as needed using pop-create.
Once autogenerated states are tested, they should be moved from idem_azure/states/azure to idem_azure/states/azure
"""


def __init__(hub):
    # Provides the ctx argument to all execution modules
    # which will have profile info from the account module
    hub.states.azure.ACCT = ["request", "azure"]
